#
#  Export an expression to a txt file.
#
import bpy,os,math
from mathutils import Vector
from bpy.types import Operator
from bpy_extras.io_utils import ImportHelper
from bpy.props import StringProperty
r2d = 180.0/math.pi

def export_pose(filename,arm_ob):
    print('Exporting Pose.')
    pbones = arm_ob.pose.bones
    fd = open(filename,'w')
    for pbone in pbones:
        if ('fa_' in pbone.name) == False:
            pbone.rotation_mode = 'XYZ'
            r = pbone.rotation_euler
            pbone.rotation_mode = 'QUATERNION'
            print('export ',pbone.name,r[0],r[1],r[2])
            fd.write('%s\t%10.6f\t%10.6f\t%10.6f\n'%(pbone.name,r2d*r[0],r2d*r[1],r2d*r[2]))
    fd.close()
            
def main(filename):
    scn = bpy.data.scenes[0]
    arms = bpy.context.selected_objects
    arm_ob = arms[0]
    if arm_ob.type == 'ARMATURE':
        export_pose(filename,arm_ob)
    else:
        print('No armature selected')
class TheMoviesPoseExporter(Operator, ImportHelper):
    """Export a TM body pose"""
    bl_idname = "object.pose_export"   #
    bl_label = "Export TM Pose"
    filename_ext = "*.pose"
    filter_glob = StringProperty(
        default="*.pose",
        options={'HIDDEN'},
        maxlen=255,
    )
    def execute(self,context):
        main(self.filepath)
        return {'FINISHED'}
def menu_func_export(self,context):
    self.layout.operator(TheMoviesPoseExporter.bl_idname,text="Export Pose (*.pose)"
)
def register():
    bpy.utils.register_class(TheMoviesPoseExporter)
    bpy.types.INFO_MT_file_export.append(menu_func_export)
def unregister():
    bpy.utils.unregister_class(TheMoviesPoseExporter)
    bpy.types.INFO_MT_file_export.remove(menu_func_export)
if __name__ == "__main__":
    register()
