# Retime a bone animation.
# This version is for multiple armatures.
# Version: 6/8/2023
#
# Usage:
#    1.0  Select the armature(s) to be affected.
#    2.0  In the timeline, select the new end frame for the animation.
#    3.0  Run this script.
#
#   Warning:
#     The more bones and the more frames, the longer the script will
#     require to exomplete the task.  Be patient.
#
import bpy,mathutils
from mathutils import *
def main():
    global framecount,new_framecount,scn,df
    arms = bpy.context.selected_objects
    scn = bpy.data.scenes[0]
#    framecount =scn['Frame Count']
    f1 = scn.frame_start
    f2 = scn.frame_end
    framecount = f2-f1+1
    f3 = scn.frame_current
    scn.frame_end = f3
    new_framecount = f3-f1+1
#    scn['Frame Count'] = new_framecount
#    df = int(float(framecount)*2.0)
    df = float(new_framecount-1)/float(framecount-1)
    for active_obj in arms:
        retime_armature(active_obj)
    scn['Frame Count'] = new_framecount
def retime_armature(active_obj):
    bonelist = []
    boneanims = []
#    active_obj = bpy.context.scene.objects.active
    if active_obj.type != 'ARMATURE':
        raise ValueError("No armature selected")
    arm_ob = active_obj
    pbones = arm_ob.pose.bones
    bonecount = len(pbones)
    print('bonecount = ',bonecount)
    print('framecount - ',framecount)
    print('new_framecount = ',new_framecount)
    for pbone in pbones:
        bonelist += [pbone]
        boneanim = []
        for f in range(framecount):
            frame = f+1
            scn.frame_current = frame
            scn.frame_set(frame)
            loc = pbone.location
            quat = pbone.rotation_quaternion
            v1 = Vector([loc[0],loc[1],loc[2]])
            if quat[0] < 0:
                q1 = Quaternion([-quat[0],-quat[1],-quat[2],-quat[3]])
            else:
                q1 = Quaternion([quat[0],quat[1],quat[2],quat[3]])
            boneanim += [[v1,q1]]
        boneanims += [boneanim]
    print('len(boneanims) = ',len(boneanims))
#    bpy.ops.anim.keyframe_clear_v3d()
#    return
    nb = 0
    for pbone in pbones:
        boneanim = boneanims[nb]
        nb = nb + 1
        for fi in range(new_framecount):
            f = fi+1
            sf = float(fi)/df
            f1 = int(sf)
            f2 = f1 + 1
            scn.frame_current = f
            scn.frame_set(f)
#            (loc1,quat1)  = boneanims[f1]
#            (loc2,quat2) = boneanims(f2)
            (loc,quat) = interpolater(fi,sf,boneanim)
            pbone.location = loc
            pbone.rotation_quaternion = quat
#            print('bonanim = ',loc,quat)
            pbone.keyframe_insert('rotation_quaternion')
            pbone.keyframe_insert('location')
#            scn.frame_current = sf+1
#            scn.frame_set(sf+1)
#            pbone.location = loc
#            pbone.rotation_quaternion = quat
#           pbone.keyframe_insert('rotation_quaternion')
#            pbone.keyframe_insert('location')
#    scn.frame_end = df
#    scn.frame_set(df)
    print('df = ',df)

def interpolater(fi,sf,boneanim):
    f1 = int(sf)
    f2 = f1+1
    if f2 == framecount:
        f2 = f1
#    print('fi = ',fi,'f1 = ',f1,' f2 = ',f2,' sf = %8.3f'%sf)
    (loc1,q1) = boneanim[f1]
    (loc2,q2) = boneanim[f2]
    q = [0,0,0,0]
    loc = [0,0,0]
    for i in range(4):
        q[i] = q1[i] + (sf-f1)*(q2[i]-q1[i])
    for i in range(3):
        loc[i] = loc1[i] + (sf-f1)*(loc2[i]-loc1[i])
    return (loc,q)

class AnimationRetimer(bpy.types.Operator):
    """Retime an animation"""
    bl_idname = "object.retimer"
    bl_label = "Animation Retimer"
    bl_info = "set current frame to new end frame"
    def execute(self,context):
        main()
        return {'FINISHED'}
def register():
    bpy.utils.register_class(AnimationRetimer)
    print('Installed: animation retimer')
if __name__ == '__main__':
    register()
                
