import bpy
import mathutils
from mathutils import *

def main():
    scn = bpy.data.scenes[0]
    f1 = scn.frame_start
    f2 = scn.frame_end
    framecount = f2-f1+1
    frames = f2-f1
    ncycles = int(scn.frame_current/frames)

    active_obj = bpy.context.scene.objects.active
    if active_obj.type != 'ARMATURE':
        raise ValueError("No armature selected") 
    arm_ob = active_obj
    pbones = arm_ob.pose.bones
    # create a dictionary with poses for basic animation
    boneanims = []
    nframes = framecount - 1
    for pbone in pbones:
        boneanim = []
        for cycle in range(ncycles):
            for frame in range(nframes):
                scn.frame_set(f1+frame)            
                scn.frame_current = f1+frame
                (x,y,z) = pbone.location
                loc = Vector([x,y,z])
                (q,x,y,z) = pbone.rotation_quaternion
                quat = Quaternion((q,x,y,z))
                boneanim += [(loc,quat)]

#        boneanim2 = []
#        for frame in range(framecount):
#            (x,r) = boneanim[frame]
#            boneanim2 += [(x,r)]
#        boneanim.extend(boneanim2)
        boneanims += [boneanim]
       
    nb = 0
    for nb in range(len(pbones)):
        pbone = pbones[nb]
        boneanim = boneanims[nb]
        print(pbones[nb].name)
        print('number frames = ',len(boneanim))
        for frame in range(len(boneanim)):
            (x,r) = boneanim[frame]
            print(frame,x,r)
            scn.frame_set(frame+1)
            pbone.location = x
            pbone.rotation_quaternion = r
            pbone.keyframe_insert('rotation_quaternion')
            pbone.keyframe_insert('location')
        (x,r) = boneanim[0]
        scn.frame_set(len(boneanim)+1)
        pbone.location = x
        pbone.rotation_quaternion = r
        pbone.keyframe_insert('rotation_quaternion')
        pbone.keyframe_insert('location')    
class AnimationExtender(bpy.types.Operator):
    """Extend a cyclic animation"""
    bl_idname = "object.extender"
    bl_label = "Animation Extender"
    bl_info = "set current frame to new end frame"
    def execute(self,context):
        main()
        return {'FINISHED'}
def register():
    bpy.utils.register_class(AnimationExtender)
    print('Installed: animation extender')        
if __name__ == '__main__':
    register()
