import bpy,mathutils
from mathutils import *
def fetch_arm_obs(scn):
    arm_obs = []
    for ob in bpy.data.objects:
        if ob.type == 'ARMATURE':
            arm_obs += [ob]
#            key = 'BAOBJ'
#            if ob['BAOBJ'] == n:
#                return 
#    print('Armature ',n,' not found.')
    return arm_obs
def fetch_destination(n,scn,scale):
    (x,y,z,angle) = scn['BA_DEST'+str(n)]
    return (scale[0]*x,scale[1]*y,scale[2]*z,angle)
def fetch_origin(n,scn,scale):
    offset = scn['BA_OFFSET'+str(n)]
    eulers = scn['BA_ROTATION'+str(n)]
    mat = Euler(eulers).to_matrix().to_4x4()
    for i in range(3):
        mat[3][i] = offset[i]*scale[i]
    return mat
def print_mat(m):
    for i in range(4):
        print('%8.5f %8.5f %8.5f %8.5f'%(m[i][0],m[i][1],m[i][2],m[i][3]))
def apply_move():
#    global fd
    scn = bpy.data.scenes[-1]
    staframe = scn.frame_start
    endframe = scn.frame_end
    mp = Matrix([(1,0,0,0),(0,1,0,0),(0,0,1,0),(0,0,0,1)])
    arm_obs = fetch_arm_obs(scn)
    for n in range(len(arm_obs)):        
#        origin = animorigins[n]
#        cob = fetch_arm_ob(n,scn)
        cob = arm_obs[n]
        ob = cob.parent
#        mp = ob.matrix_local
#        print('initial location ',ob.location)
        (x1,y1,z1) = (ob.location[0],ob.location[1],ob.location[2])
#       scale = ob.scale
        scale = cob.scale
#        print('ob.name = ',ob.name)
        (dest_x,dest_y,dest_z,dest_angle) = fetch_destination(n,scn,scale)
        print('dest = ',dest_x,dest_y,dest_z,dest_angle)
        origin = fetch_origin(n,scn,scale)
        print('First Frame  obj = ',ob.name)
        print('ob.matrix_local')
        print_mat(ob.matrix_local)
        print('mp = ')
        print_mat(mp)
        print('origin')
        print_mat(origin)
        mtx = origin*mp
        mp = origin  # next object rotates wrt previous object.
        print('mtx')
        print_mat(mtx)
        (x,y,z) = (mtx[3][0],mtx[3][1],mtx[3][2])
        (u1,v1,w1) = mtx.transposed().to_euler()
        rot = (u1,v1,w1)
        scn.frame_current = staframe
        ob.location = Vector([x+x1,y+y1,z+z1])
        ob.rotation_euler = rot
        ob.keyframe_insert('rotation_euler')
        ob.keyframe_insert('location')
        scn.frame_current = endframe
        rd = Euler([0.,0.,dest_angle]).to_matrix()
        rd.to_4x4()
        dest = Matrix([[rd[0][0],rd[1][0],rd[2][0],0],
                      [rd[0][1],rd[1][1],rd[2][1],0],
                      [rd[0][2],rd[1][2],rd[2][2],0],
                      [dest_x,dest_y,dest_z,1]])
        mv = mtx*dest
        print('mv = ')
        print_mat(mv)
        (x,y,z) = (mv[3][0],mv[3][1],mv[3][2])
        (u,v,w) = mv.to_euler()
        scn.frame_current = endframe
        print('Last Frame obj = ',ob.name)
        rot = Euler([u,v,w])
        ob.location = Vector([x+x1,y+y1,z+z1])
        ob.rotation_euler = rot
        ob.keyframe_insert('location')
        ob.keyframe_insert('rotation_euler')
        fcurves = ob.animation_data.action.fcurves
        for fcurve in fcurves:
            for kf in fcurve.keyframe_points:
                kf.interpolation = 'LINEAR'
class TMApplyMove(bpy.types.Operator):
    """Apply movement and rotation for a TM animation"""
    bl_idname = 'object.apply_move'
    bl_label = 'Apply ANM Move'
    def execute(self,context):
        apply_move()
        return {'FINISHED'}
def menu_func_import(self,context):
    self.layout.operator(TMApplyMove.bl_idname,text = 'TM Apply Move')
def register():
    bpy.utils.register_class(TMApplyMove)
#    bpy.types.INFO_HT_header.append(menu_func_import)
if __name__ == '__main__':
    register()


