#!BPY

"""
Name: 'The Movies (.anm) v. 6.3'
Blender: 279b
Group: 'Export'
Tooltip: 'Export costume and prop animations to TheMovies Game Base 6.0'
"""
#import Blender
#from Blender.Mathutils import *
import os,math,struct
import bpy,mathutils
from mathutils import *
r2d = 180.0/math.pi
d2r = math.pi/180.0
bns = []
bqr = []
locs = []
bplist = {}
bone_parent_list = []
#  Orieintaton of root bone's parent
mr = Matrix([[0.,0.,1.],
             [1.,0.,0.],
             [0.,1.,0.]])
mrt = Matrix([[0.,1.,0.],
              [0.,0.,1.],
              [1.,0.,0.]])
print('Installed: tm_anm_export 3.3')
class TheMoviesAnmExporter(bpy.types.Operator):
    """Export a TM animation"""
    bl_idname = "object.animation_export"
    bl_label = "Export"    
 
    filepath = bpy.props.StringProperty(subtype="FILE_PATH")
 
    my_float = bpy.props.FloatProperty(name="Float")
    my_bool = bpy.props.BoolProperty(name="Toggle Option")
    my_string = bpy.props.StringProperty(name="String Value")
 
    def execute(self, context):
        main(self.filepath)
        return {'FINISHED'}
 
    def invoke(self, context, event):
        context.window_manager.fileselect_add(self)
        return {'RUNNING_MODAL'}
 
    def draw(self, context):
        layout = self.layout
        col = layout.column()
        col.label(text="Custom Interface!")
 
        row = col.row()
        row.prop(self, "my_float")
        row.prop(self, "my_bool")
 
        col.prop(self, "my_string")
 
def menu_func_import(self,context):
    self.layout.operator(TheMoviesAnmExporter.bl_idname, text = "Export The Movies Animation (*.anm)")
def register():
    bpy.utils.register_class(TheMoviesAnmExporter)
    bpy.types.INFO_MT_file_export.append(menu_func_import)

#scn = Blender.Scene.GetCurrent()
#scn = bpy.data.scenes[0]    # assumes only 1 scene in blend.
def quat_mult(r,q):
    p0 = r[0]*q[0]-r[1]*q[1]-r[2]*q[2]-r[3]*q[3]
    p1 = r[0]*q[1]+r[1]*q[0]-r[2]*q[3]+r[3]*q[2]
    p2 = r[0]*q[2]+r[1]*q[3]+r[2]*q[0]-r[3]*q[1]
    p3 = r[0]*q[3]-r[1]*q[2]+r[2]*q[1]+r[3]*q[0]
    p = Quaternion((p0,p1,p2,p3))
    return p
def generate_qbrs(arm):
    global bns,bqr,locs,bplist,bone_parent_list
    bns = []
    bqr = []
    locs = []
    bone_parent_list = []
    scn = bpy.data.scenes[0]
#    arm = scn.objects.active
#for b in arm.data.bones:
    for key in arm['BoneOrder'].keys():
        bname = arm['BoneOrder'][key]
        b = arm.data.bones[bname]
        mat_local = b.matrix_local.transposed()
        print('\nb.name',b.name)
        print('b.parent.name = ',b.parent)
#        print(mat_local)
        m = mat_local
        mat = Matrix([[m[0][0],m[0][1],m[0][2]],
                     [m[1][0],m[1][1],m[1][2]],
                     [m[2][0],m[2][1],m[2][2]]]).transposed()
#        print('mat = ')
#        print(mat)
        loc = [mat_local[3][0],mat_local[3][1],mat_local[3][2]]
        quat = Matrix(mat).transposed().to_quaternion()   
#        print('quat =  ',quat)
#        print('loc = ',loc)               
#        print('key : ',key)
        bns += [b.name]
        bplist[b.name]= {}
        root_bone = False
#        if b.name == 'root':
        if b.parent == None:
            root_bone = True
            bplist[b.name]['parent'] = 'world'
        else:
            bparent = b.parent
            print('bparent = ',bparent.name)
            parent_index = bns.index(bparent.name)
            print('parent_index = ',parent_index)
            bplist[b.name]['parent'] = bns[parent_index]
        bplist[b.name]['quat'] = quat
        bplist[b.name]['mat'] = mat
        bplist[b.name]['location'] = loc
#
# Data generated
# compute qbrs 
#        print('bns = ',bns)
    bn  = 0
    rel = Vector([0,0,0])
    while bn < len(bns):
        root_bone = False
        if bn == 0:
            root_bone = True
        name = bns[bn]
        parent_index = -1
        if bn > 0:
            parent_name = bplist[bns[bn]]['parent']
            parent_index = bns.index(parent_name)
        bone_parent_list += [parent_index]
        q = bplist[name]['quat']
        mat = bplist[name]['mat']
        loc = bplist[name]['location']
#        if name == 'root':
        if root_bone:
            qref = mr.to_quaternion().conjugated()
            locp = Vector([0,0,0])
        else:
            parent = bplist[name]['parent']
            qref = bplist[parent]['quat'].conjugated()
            locp = bplist[parent]['location']
#    qstar = q.conjugated()
        qstar = q
        qbr = quat_mult(qref,qstar)
        for i in range(3):
            rel[i] = loc[i] - locp[i]
#        if name == 'root':
        if root_bone:
            pos = Vector([rel[0],rel[1],rel[2]])
        else:
            mparent = mrt*qref.conjugated().to_matrix()
            pos = mparent*Vector([rel[0],rel[1],rel[2]])
#        print('\n',name)
#        print('bqr = %10.7f %10.7f %10.7f %10.7f'%(qbr[0],qbr[1],qbr[2],qbr[3]))
#        print('    locs = %10.7f %10.7f %10.7f'%(pos[0],pos[1],pos[2]))
        bqr += [qbr]
        locs += [pos]
        bn += 1
def tm_to_blender(x,y,z):
	ea = Blender.Mathutils.Euler(0.0,0.0,y)
	mz = ea.toMatrix()
	ea = Blender.Mathutils.Euler(z,0.0,0.0)
	mx = ea.toMatrix()
	ea = Blender.Mathutils.Euler(0.0,x,0.0)
	my = ea.toMatrix()
	m = my*mz*mx
	quat = m.toQuat()
	return quat
def blender_to_tm(q):
#    m = q.toMatrix()
    m = q.to_matrix().transposed()
    x = math.atan2(m[2][0],m[0][0])/d2r
# domain check.  Beware of asin(x)
    if m[1][0] > 1.0:
        m[1][0] = 1.0
    elif m[1][0] < -1.0:
        m[1][0] = -1.0	
    y = math.asin(-m[1][0])/d2r
    z = math.atan2(m[1][2],m[1][1])/d2r
    return (x,y,z)
def UncompressU16(val):
    return (val/32767.0 - 1)
def CompressUI16(val):
    (b1,b2) = float_to_UI(val)
    return b1+256*b2
def read_item(struct_type, stream):
    value = struct.unpack(struct_type,
        stream.read(struct.calcsize(struct_type)))
    if len(value) == 1:
        value = value[0]
    return value
def read_nts(stream, length=32):
    print('read_nts at',stream.tell())
    s = stream.read(length)
    print('s = ',s)
    return s.decode("ASCII")
def degrees_to_UI(flt):
	flt = flt/180.0
	val = (flt + 1)*32767
	b2 = int(val/256)
	b1 = int(val - 256*b2)
	return (b1,b2)
def angle_range_check(flt):
    theta = flt
#    while theta < -180.0:
#        theta += 360.0
    theta = flt*d2r
    s = math.sin(theta)
    c = math.cos(theta)
    theta = math.atan2(s,c)
    return theta*r2d
def float_to_UI(flt):
#    print 'float to UI'
#    print  'flt = ',flt
    if flt > 1.0:
        flt = 1.0
    if flt < -1.0:
        flt = -1.0
    val = int((flt + 1)*32767)
#    print 'val = ',val
    b2 = int(val/256)
#    print 'b2 = ',b2
    b1 = int(val-256*b2)
#    print 'b1 = ',b1
    return (b1,b2)
#
# Find next 4 byte boundary in input stream
#
def in_pad(stream):
    align = stream.tell()
    if ((align % 4)>0):
        np = 4 - (align % 4)
        stream.read(np)
def out_pad(stream):
    align = stream.tell()
    if ((align % 4) > 0):
        np = 4 - (align % 4)
        for i in range(np):
            stream.write(struct.pack("B",0))
class AnmHeader:
    def __init__(self,stream):
        (self.Magic,) = struct.unpack('L',stream.read(4))
        (self.count_anims,) = struct.unpack('B',stream.read(1))
        (self.count_static_anims,) = struct.unpack('B',stream.read(1))
        (self.count_frames,) = struct.unpack('H',stream.read(2))
        (m11,m12,m13) = struct.unpack('fff',stream.read(12))
        (m21,m22,m23) = struct.unpack('fff',stream.read(12))
        (m31,m32,m33) = struct.unpack('fff',stream.read(12))
        (x,y,z) = struct.unpack('fff',stream.read(12))
        self.rot = Matrix(([m11,m12,m13],[m21,m22,m23],[m31,m32,m33]))
        self.loc = Vector((x,y,z))        
        (self.flags,) = struct.unpack('Bxxx',stream.read(4))
#
# metehod to copy header out to anm file after updating from ID property
# browser.
#
    def save(self,stream):
        scn = bpy.data.scenes[0]
        if 'ANMHEAD' in scn:
            yaw = scn['ANMHead']['Origin0']['yaw']
            pitch = scn['ANMHead']['Origin0']['pitch']
            roll = scn['ANMHead']['Origin0']['roll']
            x_origin = scn['ANMHead']['Origin0']['x_origin']
            y_origin = scn['ANMHead']['Origin0']['y_origin']
            z_origin = scn['ANMHead']['Origin0']['z_origin']
            eulers = Euler(roll,pitch,yaw)
            self.rot = eulers.toMatrix()
            self.loc = Vector(x_origin,y_origin,z_origin)
        bytestring = struct.pack('L',self.Magic)
        bytestring += struct.pack('B',self.count_anims)
        bytestring += struct.pack('B',self.count_static_anims)
        bytestring += struct.pack('H',self.count_frames)
        bytestring += struct.pack('fff',self.rot[0][0],self.rot[0][1],self.rot[0][2])
        bytestring += struct.pack('fff',self.rot[1][0],self.rot[1][1],self.rot[1][2])
        bytestring += struct.pack('fff',self.rot[2][0],self.rot[2][1],self.rot[2][2])
        bytestring += struct.pack('fff',self.loc[0],self.loc[1],self.loc[2])
        bytestring += struct.pack('Bxxx',self.flags)
        stream.write(bytestring)
class Origin:
    def __init__(self,stream,n_anm):
        in_pad(stream)
        (m11,m12,m13) = struct.unpack('fff',stream.read(12))
        (m21,m22,m23) = struct.unpack('fff',stream.read(12))
        (m31,m32,m33) = struct.unpack('fff',stream.read(12))
        (x,y,z) = struct.unpack('fff',stream.read(12))
        self.rot = Matrix([m11,m12,m13],[m21,m22,m23],[m31,m32,m33])
        self.loc = Vector(x,y,z)
        scn = Blender.Scene.GetCurrent()
        if scn.properties.has_key('ANMHead'):
            name = 'Origin'+str(n_anm+1)
            yaw = scn.properties['ANMHead'][name]['yaw']
            pitch = scn.properties['ANMHead'][name]['pitch']
            roll = scn.properties['ANMHead'][name]['roll']
            x_origin = scn.properties['ANMHead'][name]['x_origin']
            y_origin = scn.properties['ANMHead'][name]['y_origin']
            z_origin = scn.properties['ANMHead'][name]['z_origin']            
            eul =Euler(roll,pitch,yaw)
            self.rot = eul.toMatrix()
            self.loc = Vector(x_origin,y_origin,z_origin)
    def save(self,stream):
        bytestring = struct.pack('fff',self.rot[0][0],self.rot[0][1],self.rot[0][2])
        bytestring += struct.pack('fff',self.rot[1][0],self.rot[1][1],self.rot[1][2])
        bytestring += struct.pack('fff',self.rot[2][0],self.rot[2][1],self.rot[2][2])
        bytestring += struct.pack('fff',self.loc[0],self.loc[1],self.loc[2])
        stream.write(bytestring)
        
class BoneAnmHeader:
    def __init__(self,stream,nanm):
        in_pad(stream)
        (self.frame_count,self.bone_count) = struct.unpack('LH',stream.read(6))
        (self.besson_type,self.flags) = struct.unpack('BB',stream.read(2))
        (x,y,z,a) = struct.unpack('ffff',stream.read(16))
        self.dest = (x,y,z,a)
        (self.probability,self.compression) = struct.unpack('ff',stream.read(8))
        print('BoneAnmHeader stream.tell() = ',stream.tell())
        self.bone_list = []
        for b in range(self.bone_count):
            self.bone_list += struct.unpack('b',stream.read(1))
        in_pad(stream)
        print('bonelist = ',self.bone_list)
    def save(self,stream):
        out_pad(stream)
        bytestring = struct.pack('LH',self.frame_count,self.bone_count)
        bytestring += struct.pack('BB',self.besson_type,self.flags)
        (x,y,z,a) = self.dest
        bytestring += struct.pack('ffff',x,y,z,a)
        (prob,compress) = (self.probability,self.compression)
        bytestring += struct.pack('ff',prob,compress)
        for i in range(self.bone_count):
            bytestring += struct.pack('b',self.bone_list[i])
        stream.write(bytestring)
#        print 'bone list out to ',stream.tell()
        out_pad(stream)
#        print 'after bone list pad to ',stream.tell()
class BoneFrames:
    def __init__(self,stream,framecount,compression):
        self.bf = [0]  # first frame contains a mask, default is zero.
        fc = framecount
        if fc > 0:
            (mask,) = struct.unpack('H',stream.read(2))
        while fc > 0:
            holdcount = 1
            (data,) = struct.unpack('H',stream.read(2))
            if (data & mask):
                v1 = data & (mask-1)
                v2 = data - mask
                v3 = ~(mask-1)
                holdcount = v1 | ((v2 & v3) >> 1)
                if (holdcount>framecount) or (holdcount<0):
                    print('holdcount = ',holdcount)
                    print('location = ',stream.tell())
                    raise ValueError('value is outside bound for hold.')
                holdvalue = compression*UncompressU16(read_item('H',stream))
                for hold in range(holdcount):
                    self.bf.append(holdvalue)
            else :
                self.bf.append(compression*UncompressU16(data))
            fc -= holdcount
        print('loaded bone frames to ',stream.tell())
    def save(self,stream,framecount):
        mask = self.bf[0]
        bytestring = struct.pack('H',0)
        stream.write(bytestring)
        for f in range(framecount):
            flt = self.bf[f+1]
            (b1,b2) = float_to_UI(flt)
            data = b1+256*b2
            bytestring = struct.pack('H',data)
            stream.write(bytestring)
                    
class BoneAnimation:
    def __init__(self,stream,framecount,ba,compression):
        print('Starting Bone Animation at ',stream.tell())
        (self.trans_static,) = struct.unpack('bxxx',stream.read(4))
        print('trans_static = ',self.trans_static)
        self.x = BoneFrames(stream,framecount,180.0)
        self.y = BoneFrames(stream,framecount,180.0)
        self.z = BoneFrames(stream,framecount,180.0)
        print('read roation data to ',stream.tell())
        in_pad(stream)
        print('pad to ',stream.tell())
        if self.trans_static == 0:
            print('loading u')
            self.u = BoneFrames(stream,framecount,compression)
            print('loading v')
            self.v = BoneFrames(stream,framecount,compression)
            print('loading w')
            self.w = BoneFrames(stream,framecount,compression)
        else :
            if ba > 0:
                (self.u,) = struct.unpack('f',stream.read(4))
                (self.v,) = struct.unpack('f',stream.read(4))
                (self.w,) = struct.unpack('f',stream.read(4))
            else :
                self.u = BoneFrames(stream,0,1.0)
                self.u.bf = [read_item('f', stream)] * (framecount+1)
                self.v = BoneFrames(stream,0,1.0)
                self.v.bf = [read_item('f', stream)] * (framecount+1)
                self.w = BoneFrames(stream,0,1.0)
                self.w.bf = [read_item('f', stream)] * (framecount+1)
                self.trans_static = 0
#        print 'read location data'
#        print 'file position = ',stream.tell()
        in_pad(stream)
        print('pad to ',stream.tell())
    def save(self,stream,framecount):
        bytestring = struct.pack('Bxxx',self.trans_static)
        stream.write(bytestring)
        self.x.save(stream,framecount)
        self.y.save(stream,framecount)
        self.z.save(stream,framecount)
        out_pad(stream)
        if self.trans_static == 0:
            self.u.save(stream,framecount)
            self.v.save(stream,framecount)
            self.w.save(stream,framecount)
        else :
            bytestring = struct.pack('fff',self.u,self.v,self.w)
            stream.write(bytestring)
        out_pad(stream)        
class Animation:
    def __init__(self,stream,nanm):
        self.header = BoneAnmHeader(stream,nanm)
        compression = self.header.compression
        self.bone_animations = []
        for ba in range(self.header.bone_count):
            framecount = self.header.frame_count
            self.bone_animations += [BoneAnimation(stream,framecount,ba,compression)]
    def save(self,stream):
        self.header.save(stream)
        for ba in range(self.header.bone_count):
            self.bone_animations[ba].save(stream,self.header.frame_count)
        
class prop_anm_header(object):
    def __init__(self, stream):
        val = read_item("Hxx", stream)
        self.is_carriable = val & 1
        self.has_scale = (val & 2)>>1
        self.index_like_me = ((val | 32707) - 32707) >> 2 #This is 4 bits wide.. 0-15 valid values
        self.is_more_than_one_like_me = (val & 64) >> 6
        self.index_skeleton = (val | 127) >> 7 #This is the remainder of 8 bits wide...0-255 valid values
        self.flags = val
        if self.has_scale:
            self.compression = read_item("f", stream)
    def save(self,stream):
        bytestring = struct.pack("Hxx",self.flags)
        if self.has_scale:
            bytestring += struct.pack('f',self.compression)
        stream.write(bytestring)

class PropFrame(object):
    def __init__(self,stream):
#        self.x = 0
#        self.y = 0
#        self.z = 0
#        self.u = 0
#        self.v = 0
#        self.w = 0
        self.x = UncompressU16(read_item("H",stream))
        self.y = UncompressU16(read_item("H",stream))
        self.z = UncompressU16(read_item("H",stream))
        self.u = UncompressU16(read_item("H",stream))
        self.v = UncompressU16(read_item("H",stream))
        self.w = UncompressU16(read_item("H",stream))
    def save(self,stream):
#        print 'float to UI'
#        print 'x  = ',CompressUI16(self.x)
#        print 'y  = ',CompressUI16(self.y)
#        print 'z  = ',CompressUI16(self.z)
#        print 'u  = ',CompressUI16(self.u)
#        print 'v  = ',CompressUI16(self.v)
#        print 'w  = ',CompressUI16(self.w)
#        bytestring = struct.pack("HHHHHH",CompressUI16(self.x),
#                                 CompressUI16(self.y),
#                                 CompressUI16(self.z),
#                                 CompressUI16(self.u),
#                                 CompressUI16(self.v),
#                                 CompressUI16(self.w))
        vals = (self.x,self.y,self.z,self.u,self.v,self.w)
        for val in vals:
            (b1,b2) = float_to_UI(val)
            data = b1+256*b2
            bytestring = struct.pack('H',data)
            stream.write(bytestring)        
#        stream.write(bytes(str(bytestring).encode("utf-8")))
        
             
class PropFrames(list):
    def __init__(self,stream,framecount):
        PropPos = [None] * framecount
        for j in range(framecount):
            PropPos[j] = PropFrame(stream)
        super(PropFrames, self).__init__(PropPos)

    
class PropAnim(object):
    def __init__(self, stream, framecount):
        self.name = read_nts(stream)
        self.header = prop_anm_header(stream)
        self.frames = PropFrames(stream,framecount)
    def save(self,stream):
        bytestring = self.name
        while len(bytestring)<32:
            bytestring += chr(0)
        print('saving ',self.name)
        print('length = ',len(bytestring))
        stream.write(bytes(str(bytestring).encode("utf-8")))
        self.header.save(stream)
        for i in range(len(self.frames)):
            self.frames[i].save(stream)
#
# Compute a suitable compression factor for root data.
#
def compress_bone_anim(frame_count,poses):
    scn = bpy.data.scenes[0]
    pbone = poses['root']

    fmax = 0.0
    for f in range(frame_count):
        curframe = scn.frame_current
        scn.frame_current = f + 1
#        Blender.Set('curframe',f+1)
        rel = pbone.location
        quat = pbone.rotation_quaternion
        pmt = quat.to_matrix()
        pmt.transpose()
        pm = pmt*pbone.matrix.to_3x3()
        pm.transpose()
        loc = pm*rel
        u = loc[0] + locs[0][0]
        v = loc[1] + locs[0][1]
        w = loc[2] + locs[0][2]
        if math.fabs(u) > fmax:
            fmax = math.fabs(u)
        if math.fabs(v) > fmax:
            fmax = math.fabs(v)
        if math.fabs(w) > fmax:
            fmax = math.fabs(w)
    compression = 1.001*fmax
    print('compression = ',compression)
    return compression
def write_matrix(fd,lbl,m):
    fd.write(' lbl = %s\n'%lbl)
    for i in range(3):
        fd.write(' %8.6f %8.6f %8.6f\n'%(m[i][0],m[i][1],m[i][2]))
def update_from_timeline(anm,arm_ob):
    scn = bpy.data.scenes[0]
#    bonelist.bone_name_list(anm.header.bone_count)
    generate_qbrs(arm_ob)
    pose = arm_ob.pose
    pbones = arm_ob.pose.bones
    poses = {}
    for pbone in pbones:
        poses[pbone.name] = pbone
    for bone in range(anm.header.bone_count):
        anm.header.bone_list[bone] = bone_parent_list[bone]
    compression = compress_bone_anim(anm.header.frame_count,poses)
    print('original compression = ',anm.header.compression)
    if compression < anm.header.compression:
        compression = anm.header.compression
    anm.header.compression = compression
#    fd = open('C:/TMBlender/Ble279/export_debug.txt','w')
    for ba in range(anm.header.bone_count):
#    for ba in range(3):
        pbone = poses[bns[ba]]
        banm = anm.bone_animations[ba]
        qbr = Quaternion((bqr[ba][0],bqr[ba][1],bqr[ba][2],bqr[ba][3]))
        qstar = qbr
        qstar.conjugate()            
        for f in range(anm.header.frame_count):
#            curframe = scn.frame_current
            scn.frame_current = f+1
            scn.frame_set(f+1)
            quat = pbone.rotation_quaternion
            rel = pbone.location
            qtmg = quat_mult(quat,qstar)
            (x,y,z) = blender_to_tm(qtmg)
            pm1 = quat.to_matrix()
            pm2 = pbone.matrix.to_3x3().transposed()
            pm = pm1*pm2
#            if f < 3:
#                fd.write(' %5d %s\n'%(f+1,pbone.name))
#                write_matrix(fd,'pm1 = quat.toMatrix().transpose()',pm1)
#                write_matrix(fd,'pm2 = pbone.poseMatrix().rotationPart()',pm2)
#
#                write_matrix(fd,'pm = ',pm)
#                fd.write('x,y,z = %8.6f %8.6f %8.6f\n'%(x,y,z))
#                fmt = 'qtmg = %8.f %8.6f %8.6f %8.6f\n'
#                fd.write(fmt%(quat[0],quat[1],quat[2],quat[3]))
            loc = Vector((0,0,0))
            if pbone.name == 'root':
                loc = pm.transposed()*rel
            elif pbone.name == 'control2_arm_r' or pbone.name == 'control2_arm_l':
                loc = Vector((rel[0],-rel[1],-rel[2]))
#            loc = pm*rel
            (u,v,w) = (loc[0] + locs[ba][0],
                       loc[1] + locs[ba][1],
                       loc[2] + locs[ba][2])
            banm.x.bf[f+1] = x/180.0
            banm.y.bf[f+1] = y/180.0
            banm.z.bf[f+1] = z/180.0
            if banm.trans_static == 0:
                ua = banm.u.bf[f+1]
                va = banm.v.bf[f+1]
                wa = banm.w.bf[f+1]
                banm.u.bf[f+1] = u/compression
                banm.v.bf[f+1] = v/compression
                banm.w.bf[f+1] = w/compression
            else :
                (ua,va,wa) = (banm.u,banm.v,banm.w) 
                (banm.u,banm.v,banm.w) = (u,v,w)            
#    fd.close()                
def get_empty(name_in):
    scn = bpy.data.scenes[0]
    name = name_in.rstrip('\x00')
#    if scn.properties.has_key('Static Props') == False:
#        scn.properties['Static Props'] = {}
#    scn.properties['Static Props'][name] = 0
#    for ob in scn.objects:
#        print('ob.type = ',ob.type,' name = ',ob.name)
#        if ob.type == 'Empty':
#            if ob.properties.has_key('TheMovies') == True:
#                print('Object name = ',ob.name)
#                print('group name = ',name)
#                grpName = ob.properties['TheMovies']['grpName']
#                if grpName == name:
#                    print('found match ',ob.name)
#                    return ob
    for ob in scn.objects:
#        print 'ob.type = ',ob.type,' name = ',ob.name
        if ob.type == 'EMPTY':
            if 'TheMovies' in ob:
                if 'grpName' in ob:
#                    print('Object name = ',ob.name)
#                    print('group name = ',name,'#')

                    grpName = ob['grpName']
#                    print('grpName = ',grpName,'#')
                    if grpName == name:
                        print('found match ',ob.name)
#                    scn.properties['Static Props'][name] = 1
                        return ob
#    return scn.objects.new('Empty',name)
    return 0
def prop_update_from_timeline(p_anm,framecount):
    scn = bpy.data.scenes[0]
    f1 = scn.frame_start
    f2 = scn.frame_end
    framecount = f2 - f1 + 1
    prop_ob = get_empty(p_anm.name)
    if prop_ob == 0:
        print(p_anm.name.rstrip(),' not found.')
        return 
# compute compression factor for data
    comp = 0
    print('p_anm.name = ',p_anm.name)
    for f in range(framecount):
        scn.frame_set(f+1)
        (x,y,z) = prop_ob.location
#        print 'loc = ',x,y,z
        if math.fabs(x) > comp :
            comp = math.fabs(x)
        if math.fabs(y) > comp :
            comp = math.fabs(y)
        if math.fabs(z) > comp :
            comp = math.fabs(z)
    if comp < .001:
        comp = .95
    p_anm.header.compression = 1.001*comp
    comp = p_anm.header.compression
    print(p_anm.name,'  comp = ',comp)
    p_anm.frames.x = []
    p_anm.frames.y = []
    p_anm.frames.z = []
    p_anm.frames.u = []
    p_anm.frames.v = []
    p_anm.frames.w =[]
    for f in range(framecount):
        scn.frame_set(f+1)
        (x,y,z) = prop_ob.location
        (u,v,w) = prop_ob.rotation_euler
        (u1,v1,w1) = (u*r2d,v*r2d,w*r2d)
        u1 = angle_range_check(u1)
        v1 = angle_range_check(v1)
        w1 = angle_range_check(w1)
#        print('p_anm.frames[f].x=',p_anm.frames[f].x*comp)
        print('%2d %9.6f %9.6f %9.6f %9.6f %9.6f %9.6f'%(f,x,y,z,u1,v1,w1))
        p_anm.frames[f].x = x/comp
        p_anm.frames[f].y = y/comp
        p_anm.frames[f].z = z/comp
        p_anm.frames[f].u = u1/180.0
        p_anm.frames[f].v = v1/180.0
        p_anm.frames[f].w = w1/180.0
    
def main(filename):
    global obs
    scn = bpy.data.scenes[0]
#    manbat_bone_name_list()    # initialize armature data

#    arms = Blender.Armature.Get()
    obs = []
    for ob in bpy.data.objects:
        if ob.type == 'ARMATURE':
            obs += [ob]
    stream = open(filename,'rb')
    filepath = os.path.dirname(filename)
    filebase = os.path.basename(filename)
#    dbg = open(filepath+'/debug.txt','w')
    anm_header = AnmHeader(stream)
    target_frame_count = anm_header.count_frames
    f1 = scn.frame_start
    f2 = scn.frame_end
    frame_count = f2-f1+1
#    anm_header.count_frames = frame_count
    print('stream.tell() = ',stream.tell())
    n_anims = anm_header.count_anims
    origins = []
    for n in range(n_anims-1):
        origin = Origin(stream,n)
        origins += [origin]
    animations = []
    for i in range(anm_header.count_anims):
        animations += [Animation(stream,i)]
    print('A total of ',len(animations),'animations were loaded')


#
#  Replace transtatic position data with armature specific position data
#  and copy angles and root position from timeline
#
    for i in range(anm_header.count_anims):
        anm = animations[i]
        arm_ob = obs[i]
        name = 'BoneAnmHdr'+str(i+1)
        if name in scn:
            anm.header.compression = scn[name]['compression']
#            compression = anm.header.compression
        update_from_timeline(anm,arm_ob)
# Load static animations
    prop_anims = {}
    for pa in range(anm_header.count_static_anims):
        p_anm = PropAnim(stream, target_frame_count)
        prop_anims[p_anm.name] = p_anm
        print('prop_anim.name = ',p_anm.name)            
#
#  Export data to new animation file
#
    fname = filepath + '/n_'+filebase
    ostream = open(fname,'wb')
    anm_header.save(ostream)
    for i in range(anm_header.count_anims-1):
        origin = origins[i]
        origin.save(ostream)
    out_pad(ostream)
    for i in range(anm_header.count_anims):
        anm = animations[i]
        anm.save(ostream)
#
# Export prop animation data
#
#    for i in range(anm_header.count_static_anims):
    for pa in prop_anims.keys():
        p_anm = prop_anims[pa]
        prop_update_from_timeline(p_anm,anm_header.count_frames)
        p_anm.save(ostream)
    print('updated ',fname)
    print('current working diretory = ',os.getcwd())
    
#
#
#
#Blender.Window.FileSelector(main,"Update ANM",'*.anm')
#filename = 'c:/TMBlender/data/animations/high/tired_breathe_idle.anm'
#main(filename)
def anm_out():
    bpy.ops.object.animation_export('INVOKE_DEFAULT')
if __name__ == "__main__":
    register()
