#
#  Run this script to check if model is ready for export to "The Movies"
#  Look in system console for results.
#  Updated 7/5/2022
#
import bpy

def get_used_images():
    image_list = []
    for obj in bpy.data.objects:
        if obj.type == 'MESH':
            mat = obj.data.materials[0]
            for i in range(4):
                slot = mat.texture_slots[i]
                if slot != None:
                    if slot.use:
                        if slot.texture.type == 'IMAGE':
                            image_list.append(slot.texture.image)
    return image_list
            
            
            
# Groups Tests
def InGroupTest():
    global warnings
    groups = bpy.data.groups
    objects = bpy.data.objects
    group_keys = groups.keys()
    for obj in objects:
        if obj.type == 'MESH':
            if len(obj.users_group) == 0:
                testcase = '    '+obj.name + ' is not in a blender group.  Add it to a group'
                warnings += 1
                return testcase
    testcase = 'OK: Each Blender Object is in a Blender Group.'
    return testcase

def MultiGroupTest():
    global warnings
    objects = bpy.data.objects
    for obj in objects:
        if obj.type == 'MESH':
            if len(obj.users_group) > 1:
                warnings += 1
                testcase = '    '+obj.name + ' is in multiple blender groups. There should only be one'
                return testcase
    testcase = 'OK: Each blender object is in only ONE blender group.'
    return testcase            
    
def GroupHasEmptyTest():
    global warnings
    groups = bpy.data.groups
    ngrps = len(groups)
    group_list = []
    if ngrps == 0:
        warnings += 1
        return '    No groups found.  Empty pivot must be in a group.'
    for grp in groups:
        objects = grp.objects
        testcase = '    '+grp.name + 'has no empty object.  Add one if this group is to be exported.'
        has_empty = False
        for obj in objects:
            if obj.type == 'EMPTY':
                has_empty = True
                ngrps -= 1
        if has_empty == False:
            testcase = '    '+grp.name+' has no empty object.'
            warnings += 1
            return testcase
    testcase = 'OK: Each populated Blender Group has an Empty pivot object.'
    return testcase

def GroupParentNameTest():
    global warnings
    objects = bpy.data.objects
    for obj in objects:
        if obj.type == 'EMPTY':
            if obj.name == obj.users_group[0].name:
                testcase = 'OK: The name of the Group and parent Empty are the same.'
                return testcase
    testcase = '    Empty pivot '+obj.name+' is named differently thatn Blender Group.'
    warnings += 1
    return testcase
def GroupPropertyNameTest():
    global warnings
    objects = bpy.data.objects
    for obj in objects:
        if obj.type == 'EMPTY':
#            print('objname = ',obj.name)
            grpname = obj.name.split('.',1)[1]
#            print('grpname = ',grpname)
#            print('grpName = ',obj['grpName'])
            if ('grpName' in obj) == False:
                obj['grpName'] = 'Group'
            if obj['grpName'] == grpname:
                testcase = 'OK: The name of Group and property grpName are the same.'
                return testcase
    testcase = '    Object property grpName should be the same as the blender group name.'
    warnings += 1
    return testcase
def GroupHeirarchyTest():
    global warnings
    objects = bpy.data.objects
    for obj in objects:
        if obj.type == 'MESH':
            parent = obj.parent
#            print('parent = ',parent.name,' parent.type = ',parent.type)
            if (parent.type == 'ARMATURE') or (parent.type == 'EMPTY'):
                testcase = 'OK: Meshes must children of an Empty or an Armature.'
                return testcase
    warnings += 1
    testcase = '    Mesh is not parented to an armature or an empty.'
def BoneVertexGroupNameTest():
    global warnings
    for obj in bpy.data.objects:
        if obj.type == 'MESH':
            if obj.parent == None:
                testcase = '    Mesh object ',obj.name + ' has no parent.'
                return testcase
            if obj.parent.type == 'ARMATURE':
                bones = obj.parent.data.bones
#                print('bones = ',bones.keys())
                vertex_groups = obj.vertex_groups.keys()
#                print('vertex_groups = ',vertex_groups)
                for vg in vertex_groups:
                    if vg in bones.keys() == False:
                        testcase = 'Mesh '+ obj.name+' vertex groups do not match any bone names'
                        warnings += 1
                        return testcase
    testcase = 'OK: Vertex groups for mesh must match Bone names.'
    return testcase
def MaterialZeroTest():
    global warnings
    objects = bpy.data.objects
    for obj in objects:
        if obj.type == 'MESH':
            if len(obj.users_group)>0:
                if len(obj.material_slots) == 0:
                    testcase = '    '+obj.name+' has no material in the first slot'
                    warnings += 1
                    return testcase
    testcase = 'OK: Each grouped mesh has a material in the first slot'
    return testcase
def MaterialTypeTest():
    global warnings
    materials = bpy.data.materials
    for material in materials:
        for i in range(4):
            if material.texture_slots[i] != None:
                type = material.texture_slots[i].texture.type
                if type != 'NONE':
                    if material.texture_slots[i].texture.type != 'IMAGE':
                        testcase = '    Textures in materials must be images'
                        warnings += 1
                        print('type = ',material.texture_slots[i].texture.type)
                        return testcase
    testcase = 'OK: Textures in Materials are images'
    return testcase
def UnusedMaterialsTest():
    global warnings
    materials = bpy.data.materials
    for mat in materials:
        if mat.users == 0:
            warnings += 1
            testcase = '    '+mat.name+ 'has no users'
            return testcase
    testcase = 'OK: No unused materials found.'
    return testcase
def UnusedImagesTest():
    global warnings
    images = bpy.data.images
    images_list = get_used_images()
    for image in images:
        if image not in images_list:
            warnings += 1
            testcase = '    '+image.name+' not used'
            return testcase               
    testcase = 'OK: No unused images'
    return testcase
def ArmatureIDTest():
    global warnings
    for obj in bpy.data.objects:
        if obj.type == 'ARMATURE':
            if obj.name.isdigit() == False:
                testcase = '    Armature '+obj.name+ ' is not a number'
                warnings += 1
                return testcase
    testcase = 'OK: Armature name is a number'
    return testcase
def TrianglesTest():
    global warnings
    for ob in bpy.data.meshes:
        for f in ob.polygons:
            if len(f.vertices)!=3:
                testcase = '    ' + ob.name+ ' contains quads. Triangulate faces'
                warnings += 1
                return testcase
    testcase = 'OK: All faces are triangles'            
def preflight():
    global warnings
    warnings = 0
    print('')
    print(GroupHasEmptyTest())
    if warnings == 0:
        print(InGroupTest())
        print(MultiGroupTest())
        print(GroupParentNameTest())
        print(GroupPropertyNameTest())
        print(GroupHeirarchyTest())
        print(BoneVertexGroupNameTest())
        print(MaterialZeroTest())
        print(MaterialTypeTest())
        print(UnusedMaterialsTest())
        print(UnusedImagesTest())
        print(ArmatureIDTest())
        print(TrianglesTest())
    print('Warnings = ',warnings)
    print(' ')
class TMPreflightTest(bpy.types.Operator):
    """Run the TM preflight test"""
    bl_idname = "object.preflight_test"
    bl_label = "Prelight Test"
    bl_info = "Look in console for output"
    def execute(self,context):
        preflight()
        return {'FINISHED'}
def menu_func_import(self,context):
    self.layout.operator(TMPreflightTest.bl_idname, text = "Preflight")

def register():
    bpy.utils.register_class(TMPreflightTest)
#    bpy.types.INFO_HT_header.append(menu_func_import)
    print('Installed: tm_preflight')
    
if __name__ == '__main__':
    register()
    
