#!BPY

"""
Name: 'The Movies Sew Head'
Blender: 279
Group: 'Misc'
Tooltip: 'Aligns neck connect vertices for head and costume.'
"""
__author__ = ["Cary Semar"]
__version__ = "1.0  4/5/2023"
__bpydoc__ = """\
 Usage:
    Select the two objects containing neckconnect groups.
    Run this script.  The neck connect vertices in the selected object
    Are moved on top of the neck connect vertices of the active object.
    The first object you click on is the selected object.
    The second object you click on is the active object.
    Afterward, the neckconnect vertices are removed from the neck and head
    vertex groups.

    If you do not select two mesh objects, the script will raise an error.
    If you select an object that does not contain a neckconnect group, the
    script will raise an error.

    The variable threshhold is used to find corresponding neckconnect vertices
    in the mesh.  If after running the script, there is still a gap, increase
    threshhold slightly and run the script again.
    

"""
import bpy
import mathutils as M
import math
print('\nSewing on the head')
threshhold = .02  #vertices closer than this are pushed together.
def distance(vtx1,vtx2):
        v1 = vtx1.co
        v2 = vtx2.co
        dv = v2-v1
        return math.sqrt(dv*dv)
def main():
    scn = bpy.data.scenes[0]
    selected_objects = bpy.context.selected_objects
    active_obj = bpy.context.scene.objects.active
    obj2 = active_obj
    if obj2 == None:
        raise ValueError('No active object')
    if len(selected_objects) == 0:
        raise ValueError('Select two objects')

    print('Active Object:',obj2.name)
    if len(selected_objects) > 2:
        raise ValueError('Too many objects selected') 
    print('Selected Objects:',selected_objects)

    for ob in selected_objects:
        if ob != obj2:
            obj1 = ob
    print('ob1 = ',obj1.name)
    ob = obj2
    group_lookup = {g.index: g.name for g in ob.vertex_groups}
    verts2 = {name: [] for name in group_lookup.values()}
    for v in ob.data.vertices:
        for g in v.groups:
            verts2[group_lookup[g.group]].append(v.index)
    print('verts neckconnect = ',verts2['neckconnect'])
    indices2 = verts2['neckconnect']
    ob = obj1
    group_lookup = {g.index: g.name for g in ob.vertex_groups}
    verts1 = {name: [] for name in group_lookup.values()}
    for v in ob.data.vertices:
        for g in v.groups:
            verts1[group_lookup[g.group]].append(v.index)
    print('verts neckconnect = ',verts1['neckconnect'])
    indices1 = verts1['neckconnect'] 

    for i in indices1:
        vtx1 = obj1.data.vertices[i]
#    print('\nindex1 = ',i)
        min = .5
        vtx = None
        for j in indices2:
#        print('index2 = ',j)
            vtx2 = obj2.data.vertices[j]
            sep = distance(vtx1,vtx2)
            if sep < min:
                min = sep
                vtx = vtx2
        if vtx == None:
            print('No min? ',i)
        else:
            print('%5d %10.6f'%(i,min))
            (x,y,z) = (vtx.co[0],vtx.co[1],vtx.co[2])
            vtx1.co.x = x
            vtx1.co.y = y
            vtx1.co.z = z
#        print('index2 = ',j,' sep = ',sep)
#        if distance(vtx1,vtx2) < threshhold:
#            print('     vtx2 = ',vtx2.co,distance(vtx1,vtx2))
#            (x,y,z) = (vtx2.co[0],vtx2.co[1],vtx2.co[2])
#            vtx1.co.x = x
#            vtx1.co.y = y
#            vtx1.co.z = z

# Need to remove neckconnect vertices from neck and head vertex groups for
# head and body.  This should keep the neck and costume vertices from
# separating during an animation.
#
    obj2.vertex_groups['neck'].remove(indices2)
    obj1.vertex_groups['neck'].remove(indices1)
class TMSewHead(bpy.types.Operator):
    """ align neckconnect vertices """
    bl_idname = "object.sew_head"
    bl_label = "Sew Head"
    bl_info = "Look in console for output"
    def execute(self,context):
        main()
        return {'FINISHED'}
def register():
    bpy.utils.register_class(TMSewHead)
    print('Installed: tm sew head')
if __name__ == '__main__':
    register()



                

